@@ -1,9 +1,14 @@
+"""This module also includes the :py:class:`SpacePacketHeader` class, which is the header component
+of all CCSDS packets."""
 from __future__ import annotations
+
+from abc import abstractmethod, ABC
 import enum
 import struct
 
-from typing import Tuple, Deque, List, Final, Optional
-from spacepackets.log import get_console_logger
+from typing import Tuple, Deque, List, Final, Optional, Sequence
+
+from spacepackets.exceptions import BytesTooShortError
 
 SPACE_PACKET_HEADER_SIZE: Final = 6
 SEQ_FLAG_MASK = 0xC000
@@ -11,7 +16,7 @@ APID_MASK = 0x7FF
 PACKET_ID_MASK = 0x1FFF
 
 
-class PacketTypes(enum.IntEnum):
+class PacketType(enum.IntEnum):
     TM = 0
     TC = 1
 
@@ -24,6 +29,10 @@ class SequenceFlags(enum.IntEnum):
 
 
 class PacketSeqCtrl:
+    """The packet sequence control is the third and fourth byte of the space packet header.
+    It contains the sequence flags and the 14-bit sequence count.
+    """
+
     def __init__(self, seq_flags: SequenceFlags, seq_count: int):
         if seq_count > pow(2, 14) - 1 or seq_count < 0:
             raise ValueError(
@@ -38,6 +47,19 @@ class PacketSeqCtrl:
             f"seq_count={self.seq_count!r})"
         )
 
+    def __str__(self):
+        if self.seq_flags == SequenceFlags.CONTINUATION_SEGMENT:
+            seqstr = "CONT"
+        elif self.seq_flags == SequenceFlags.LAST_SEGMENT:
+            seqstr = "FIRST"
+        elif self.seq_flags == SequenceFlags.LAST_SEGMENT:
+            seqstr = "LAST"
+        elif self.seq_flags == SequenceFlags.UNSEGMENTED:
+            seqstr = "UNSEG"
+        else:
+            raise ValueError("Invalid sequence flag")
+        return f"PSC: [Seq Flags: {seqstr}, Seq Count: {self.seq_count}]"
+
     def raw(self) -> int:
         return self.seq_flags << 14 | self.seq_count
 
@@ -53,7 +75,10 @@ class PacketSeqCtrl:
 
 
 class PacketId:
-    def __init__(self, ptype: PacketTypes, sec_header_flag: bool, apid: int):
+    """The packet ID forms the last thirteen bits of the first two bytes of the
+    space packet header."""
+
+    def __init__(self, ptype: PacketType, sec_header_flag: bool, apid: int):
         if apid > pow(2, 11) - 1 or apid < 0:
             raise ValueError(
                 f"Invalid APID, exceeds maximum value {pow(2, 11) - 1} or negative"
@@ -64,7 +89,7 @@ class PacketId:
 
     @classmethod
     def empty(cls):
-        return cls(ptype=PacketTypes.TM, sec_header_flag=False, apid=0)
+        return cls(ptype=PacketType.TM, sec_header_flag=False, apid=0)
 
     def __repr__(self):
         return (
@@ -72,33 +97,66 @@ class PacketId:
             f"sec_header_flag={self.sec_header_flag!r}, apid={self.apid!r})"
         )
 
+    def __str__(self):
+        pstr = "TM" if self.ptype == PacketType.TM else "TC"
+        return (
+            f"Packet ID: [Packet Type: {pstr}, Sec Header Flag: {self.sec_header_flag}, "
+            f"APID: {self.apid:#05x}]"
+        )
+
     def raw(self) -> int:
         return self.ptype << 12 | self.sec_header_flag << 11 | self.apid
 
     @classmethod
     def from_raw(cls, raw: int) -> PacketId:
         return cls(
-            ptype=PacketTypes((raw >> 12) & 0b1),
+            ptype=PacketType((raw >> 12) & 0b1),
             sec_header_flag=bool(raw >> 11 & 0b1),
             apid=raw & APID_MASK,
         )
 
 
-class SpacePacketHeader:
+class AbstractSpacePacket(ABC):
+    @property
+    @abstractmethod
+    def apid(self) -> int:
+        pass
+
+    @property
+    @abstractmethod
+    def seq_count(self) -> int:
+        pass
+
+
+class SpacePacketHeader(AbstractSpacePacket):
     """This class encapsulates the space packet header.
     Packet reference: Blue Book CCSDS 133.0-B-2"""
 
     def __init__(
         self,
-        packet_type: PacketTypes,
+        packet_type: PacketType,
         apid: int,
         seq_count: int,
         data_len: int,
-        sec_header_flag: bool = True,
+        sec_header_flag: bool = False,
         seq_flags: SequenceFlags = SequenceFlags.UNSEGMENTED,
         ccsds_version: int = 0b000,
     ):
-        """Create a space packet header with the given field parameters
+        """Create a space packet header with the given field parameters.
+
+        >>> sph = SpacePacketHeader(packet_type=PacketType.TC, apid=0x42, seq_count=0, data_len=12)
+        >>> hex(sph.apid)
+        '0x42'
+        >>> sph.packet_type
+        <PacketType.TC: 1>
+        >>> sph.data_len
+        12
+        >>> sph.packet_len
+        19
+        >>> sph.packet_id
+        PacketId(ptype=<PacketType.TC: 1>, sec_header_flag=False, apid=66)
+        >>> sph.psc
+        PacketSeqCtrl(seq_flags=<SequenceFlags.UNSEGMENTED: 3>, seq_count=0)
 
         :param packet_type: 0 for Telemetery, 1 for Telecommands
         :param apid: Application Process ID, should not be larger
@@ -143,7 +201,7 @@ class SpacePacketHeader:
 
     def pack(self) -> bytearray:
         """Serialize raw space packet header into a bytearray, using big endian for each
-        2 octet field of the space packet header"""
+        2 octet field of the space packet header."""
         header = bytearray()
         packet_id_with_version = self.ccsds_version << 13 | self.packet_id.raw()
         header.extend(struct.pack("!H", packet_id_with_version))
@@ -175,37 +233,47 @@ class SpacePacketHeader:
     def seq_count(self):
         return self.psc.seq_count
 
+    @seq_count.setter
+    def seq_count(self, seq_cnt):
+        self.psc.seq_count = seq_cnt
+
     @property
     def seq_flags(self):
         return self.psc.seq_flags
 
+    @seq_flags.setter
+    def seq_flags(self, value):
+        self.psc.seq_flags = value
+
     @property
     def header_len(self) -> int:
         return SPACE_PACKET_HEADER_SIZE
 
+    @apid.setter
+    def apid(self, apid):
+        self.packet_id.apid = apid
+
     @property
     def packet_len(self) -> int:
-        """Retrieve the full space packet size when packed
+        """Retrieve the full space packet size when packed.
+
         :return: Size of the TM packet based on the space packet header data length field.
-        The space packet data field is the full length of data field minus one without
-        the space packet header.
         """
         return SPACE_PACKET_HEADER_SIZE + self.data_len + 1
 
     @classmethod
-    def unpack(cls, space_packet_raw: bytes) -> SpacePacketHeader:
-        """Unpack a raw space packet into the space packet header instance
+    def unpack(cls, data: bytes) -> SpacePacketHeader:
+        """Unpack a raw space packet into the space packet header instance.
+
         :raise ValueError: Raw packet length invalid
         """
-        if len(space_packet_raw) < SPACE_PACKET_HEADER_SIZE:
-            logger = get_console_logger()
-            logger.warning("Packet size smaller than PUS header size!")
-            raise ValueError
-        packet_version = (space_packet_raw[0] >> 5) & 0b111
-        packet_type = PacketTypes((space_packet_raw[0] >> 4) & 0b1)
-        secondary_header_flag = (space_packet_raw[0] >> 3) & 0b1
-        apid = ((space_packet_raw[0] & 0b111) << 8) | space_packet_raw[1]
-        psc = struct.unpack("!H", space_packet_raw[2:4])[0]
+        if len(data) < SPACE_PACKET_HEADER_SIZE:
+            raise BytesTooShortError(SPACE_PACKET_HEADER_SIZE, len(data))
+        packet_version = (data[0] >> 5) & 0b111
+        packet_type = PacketType((data[0] >> 4) & 0b1)
+        secondary_header_flag = (data[0] >> 3) & 0b1
+        apid = ((data[0] & 0b111) << 8) | data[1]
+        psc = struct.unpack("!H", data[2:4])[0]
         sequence_flags = (psc & SEQ_FLAG_MASK) >> 14
         ssc = psc & (~SEQ_FLAG_MASK)
         return SpacePacketHeader(
@@ -213,7 +281,7 @@ class SpacePacketHeader:
             apid=apid,
             sec_header_flag=bool(secondary_header_flag),
             ccsds_version=packet_version,
-            data_len=struct.unpack("!H", space_packet_raw[4:6])[0],
+            data_len=struct.unpack("!H", data[4:6])[0],
             seq_flags=SequenceFlags(sequence_flags),
             seq_count=ssc,
         )
@@ -221,9 +289,9 @@ class SpacePacketHeader:
     def __repr__(self):
         return (
             f"{self.__class__.__name__}(packet_version={self.ccsds_version!r}, "
-            f"packet_type={self.packet_type!r}, apid={self.apid!r}, seq_cnt={self.seq_count!r}),"
-            f"data_len={self.data_len!r}, sec_header_flag={self.sec_header_flag!r},"
-            f"seq_flags={self.seq_flags!r}"
+            f"packet_type={self.packet_type!r}, apid={self.apid!r}, seq_cnt={self.seq_count!r}, "
+            f"data_len={self.data_len!r}, sec_header_flag={self.sec_header_flag!r}, "
+            f"seq_flags={self.seq_flags!r})"
         )
 
     def __eq__(self, other: SpacePacketHeader):
@@ -293,7 +361,7 @@ class SpacePacket:
 
 
 def get_space_packet_id_bytes(
-    packet_type: PacketTypes,
+    packet_type: PacketType,
     secondary_header_flag: True,
     apid: int,
     version: int = 0b000,
@@ -318,7 +386,7 @@ def get_space_packet_id_bytes(
 
 
 def get_sp_packet_id_raw(
-    packet_type: PacketTypes, secondary_header_flag: bool, apid: int
+    packet_type: PacketType, secondary_header_flag: bool, apid: int
 ) -> int:
     """Get packet identification segment of packet primary header in integer format"""
     return PacketId(packet_type, secondary_header_flag, apid).raw()
@@ -348,14 +416,16 @@ def get_total_space_packet_len_from_len_field(len_field: int):
 
 
 def parse_space_packets(
-    analysis_queue: Deque[bytearray], packet_ids: Tuple[int]
+    analysis_queue: Deque[bytearray], packet_ids: Sequence[PacketId]
 ) -> List[bytearray]:
     """Given a deque of bytearrays, parse for space packets. Any broken headers will be removed.
-    If a packet is detected and the broken tail packets will be reinserted into the given deque
+    If a packet is detected and the broken tail packets will be reinserted into the given deque.
+
     :param analysis_queue:
     :param packet_ids:
     :return:
     """
+    ids_raw = [packet_id.raw() for packet_id in packet_ids]
     tm_list = []
     concatenated_packets = bytearray()
     if not analysis_queue:
@@ -374,7 +444,7 @@ def parse_space_packets(
             struct.unpack("!H", concatenated_packets[current_idx : current_idx + 2])[0]
             & PACKET_ID_MASK
         )
-        if current_packet_id in packet_ids:
+        if current_packet_id in ids_raw:
             result, current_idx = __handle_packet_id_match(
                 concatenated_packets=concatenated_packets,
                 analysis_queue=analysis_queue,
