@@ -1,9 +1,11 @@
 from __future__ import annotations
-from typing import Tuple, Optional, TypeVar, Type, Union, List
+
+from abc import ABC, abstractmethod
+from typing import Tuple, Optional, Type, Union, List, Any, cast
 import enum
-from spacepackets.log import get_console_logger
 from spacepackets.cfdp.lv import CfdpLv
-from spacepackets.cfdp.definitions import ConditionCode, FaultHandlerCodes
+from spacepackets.cfdp.defs import ConditionCode, FaultHandlerCode
+from spacepackets.exceptions import BytesTooShortError
 
 
 class TlvTypes(enum.IntEnum):
@@ -38,8 +40,6 @@ class FilestoreResponseStatusCode(enum.IntEnum):
 
     SUCCESS = 0b0000
     NOT_PERFORMED = 0b1111
-    # Offsets past the size of the file are not supported for the copy procedure for now
-    APPEND_FROM_DATA_INVALID_OFFSET = 0b0001
     APPEND_FROM_DATA_FILE_NOT_EXISTS = 0b0010
 
     CREATE_SUCCESS = FilestoreActionCode.CREATE_FILE_SNM << 4 | SUCCESS
@@ -129,7 +129,44 @@ def map_int_status_code_to_enum(
         return FilestoreResponseStatusCode.INVALID
 
 
-class CfdpTlv:
+class TlvTypeMissmatch(Exception):
+    def __init__(self, found: TlvTypes, expected: TlvTypes, *args, **kwards):
+        self.found = found
+        self.expected = expected
+
+    def __str__(self):
+        return f"Expected TLV {self.expected}, found {self.found}"
+
+
+class AbstractTlvBase(ABC):
+    @abstractmethod
+    def pack(self) -> bytearray:
+        pass
+
+    @property
+    @abstractmethod
+    def packet_len(self):
+        pass
+
+    @property
+    @abstractmethod
+    def tlv_type(self) -> TlvTypes:
+        pass
+
+    @property
+    @abstractmethod
+    def value(self) -> bytes:
+        pass
+
+    def __eq__(self, other: AbstractTlvBase):
+        return self.tlv_type == other.tlv_type and self.value == other.value
+
+    def check_type(self, tlv_type: TlvTypes):
+        if self.tlv_type != tlv_type:
+            raise TlvTypeMissmatch(found=tlv_type, expected=self.tlv_type)
+
+
+class CfdpTlv(AbstractTlvBase):
     """Encapsulates the CFDP TLV (type-length-value) format.
     For more information, refer to CCSDS 727.0-B-5 p.77
     """
@@ -144,59 +181,75 @@ class CfdpTlv:
         :raise ValueError: Length invalid or value length not equal to specified length
         """
         self.length = len(value)
-        if self.length > 255:
-            logger = get_console_logger()
-            logger.warning("Length larger than allowed 255 bytes")
-            raise ValueError
-        self.tlv_type = tlv_type
-        self.value = value
+        if self.length > pow(2, 8) - 1:
+            raise ValueError("Length larger than allowed 255 bytes")
+        self._tlv_type = tlv_type
+        self._value = value
+
+    @property
+    def tlv_type(self):
+        return self._tlv_type
+
+    @tlv_type.setter
+    def tlv_type(self, tlv_type: TlvTypes):
+        self._tlv_type = tlv_type
+
+    @property
+    def value(self) -> bytes:
+        return self._value
 
     def pack(self) -> bytearray:
         tlv_data = bytearray()
         tlv_data.append(self.tlv_type)
         tlv_data.append(self.length)
-        tlv_data.extend(self.value)
+        tlv_data.extend(self._value)
         return tlv_data
 
     @classmethod
-    def unpack(cls, raw_bytes: bytes) -> CfdpTlv:
+    def unpack(cls, data: bytes) -> CfdpTlv:
         """Parses LV field at the start of the given bytearray
 
-        :param raw_bytes:
+        :param data:
+        :raise BytesTooShortError: Length of raw data too short.
         :raise ValueError: Invalid format of the raw bytearray or type field invalid
         :return:
         """
-        if len(raw_bytes) < 2:
-            logger = get_console_logger()
-            logger.warning("Invalid length for TLV field, less than 2")
-            raise ValueError
+        if len(data) < 2:
+            raise BytesTooShortError(2, len(data))
         try:
-            tlv_type = TlvTypes(raw_bytes[0])
+            tlv_type = TlvTypes(data[0])
         except ValueError:
-            logger = get_console_logger()
-            logger.warning(
-                f"TLV field invalid, found value {raw_bytes[0]} is not a possible TLV parameter"
+            raise ValueError(
+                f"TLV field invalid, found value {data[0]} is not a possible TLV parameter"
             )
-            raise ValueError
 
         value = bytearray()
-        if len(raw_bytes) > 2:
-            length = raw_bytes[1]
-            if 2 + length > len(raw_bytes):
-                logger = get_console_logger()
-                logger.warning(f"Detected TLV length exceeds size of passed bytearray")
-                raise ValueError
-            value.extend(raw_bytes[2 : 2 + length])
+        if len(data) > 2:
+            length = data[1]
+            if 2 + length > len(data):
+                raise BytesTooShortError(length + 2, len(data))
+            value.extend(data[2 : 2 + length])
         return cls(tlv_type=tlv_type, value=value)
 
     @property
     def packet_len(self) -> int:
-        return self.MINIMAL_LEN + len(self.value)
+        return self.MINIMAL_LEN + len(self._value)
+
+    def __repr__(self):
+        return f"{self.__class__.__name__}(tlv_type={self.tlv_type!r}, value={self.value!r})"
+
+    def __str__(self):
+        return (
+            f"CFDP TLV with type {self.tlv_type} and data 0x[{self._value.hex(sep=',')}] with "
+            f"length {len(self._value)}"
+        )
 
 
-class ConcreteTlvBase:
-    def __init__(self, tlv: CfdpTlv):
-        self.tlv = tlv
+class EntityIdTlv(AbstractTlvBase):
+    TLV_TYPE = TlvTypes.ENTITY_ID
+
+    def __init__(self, entity_id: bytes):
+        self.tlv = CfdpTlv(tlv_type=TlvTypes.ENTITY_ID, value=entity_id)
 
     def pack(self) -> bytearray:
         return self.tlv.pack()
@@ -205,60 +258,76 @@ class ConcreteTlvBase:
     def packet_len(self):
         return self.tlv.packet_len
 
-    def _check_type(self, tlv_type: TlvTypes):
-        if self.tlv.tlv_type != tlv_type:
-            raise ValueError
+    @property
+    def tlv_type(self) -> TlvTypes:
+        return EntityIdTlv.TLV_TYPE
 
-
-class EntityIdTlv(ConcreteTlvBase):
-    def __init__(self, entity_id: bytes):
-        tlv = CfdpTlv(tlv_type=TlvTypes.ENTITY_ID, value=entity_id)
-        ConcreteTlvBase.__init__(self, tlv=tlv)
+    @property
+    def value(self) -> bytes:
+        return self.tlv.value
 
     @classmethod
     def __empty(cls) -> EntityIdTlv:
         return cls(entity_id=bytes())
 
     @classmethod
-    def unpack(cls, raw_bytes: bytes) -> EntityIdTlv:
+    def unpack(cls, data: bytes) -> EntityIdTlv:
         entity_id_tlv = cls.__empty()
-        entity_id_tlv.tlv = CfdpTlv.unpack(raw_bytes=raw_bytes)
-        entity_id_tlv._check_type(tlv_type=TlvTypes.ENTITY_ID)
+        entity_id_tlv.tlv = CfdpTlv.unpack(data=data)
+        entity_id_tlv.check_type(tlv_type=TlvTypes.ENTITY_ID)
         return entity_id_tlv
 
     @classmethod
     def from_tlv(cls, cfdp_tlv: CfdpTlv) -> EntityIdTlv:
-        if cfdp_tlv.tlv_type != TlvTypes.ENTITY_ID:
-            raise ValueError
+        if cfdp_tlv.tlv_type != cls.TLV_TYPE:
+            raise TlvTypeMissmatch(cfdp_tlv.tlv_type, cls.TLV_TYPE)
         entity_id_tlv = cls.__empty()
         entity_id_tlv.tlv = cfdp_tlv
         return entity_id_tlv
 
 
-class FaultHandlerOverrideTlv(ConcreteTlvBase):
+class FaultHandlerOverrideTlv(AbstractTlvBase):
+    TLV_TYPE = TlvTypes.FAULT_HANDLER
+
     def __init__(
         self,
         condition_code: ConditionCode,
-        handler_code: FaultHandlerCodes,
+        handler_code: FaultHandlerCode,
     ):
         self.condition_code = condition_code
         self.handler_code = handler_code
-        tlv_value = bytes([(self.condition_code << 4) | self.handler_code])
-        tlv = CfdpTlv(tlv_type=TlvTypes.FAULT_HANDLER, value=tlv_value)
-        ConcreteTlvBase.__init__(self, tlv=tlv)
+        self.tlv = CfdpTlv(
+            tlv_type=self.tlv_type,
+            value=bytes([self.condition_code << 4 | self.handler_code]),
+        )
+
+    def pack(self) -> bytearray:
+        return self.tlv.pack()
+
+    @property
+    def packet_len(self):
+        return self.tlv.packet_len
+
+    @property
+    def value(self) -> bytes:
+        return self.tlv.value
+
+    @property
+    def tlv_type(self) -> TlvTypes:
+        return FaultHandlerOverrideTlv.TLV_TYPE
 
     @classmethod
     def __empty(cls) -> FaultHandlerOverrideTlv:
         return cls(
             condition_code=ConditionCode.NO_ERROR,
-            handler_code=FaultHandlerCodes.IGNORE_ERROR,
+            handler_code=FaultHandlerCode.IGNORE_ERROR,
         )
 
     @classmethod
-    def unpack(cls, raw_bytes: bytes) -> FaultHandlerOverrideTlv:
+    def unpack(cls, data: bytes) -> FaultHandlerOverrideTlv:
         fault_handler_ovr_tlv = cls.__empty()
-        fault_handler_ovr_tlv.tlv = CfdpTlv.unpack(raw_bytes=raw_bytes)
-        fault_handler_ovr_tlv._check_type(tlv_type=TlvTypes.FAULT_HANDLER)
+        fault_handler_ovr_tlv.tlv = CfdpTlv.unpack(data=data)
+        fault_handler_ovr_tlv.check_type(tlv_type=FaultHandlerOverrideTlv.TLV_TYPE)
         fault_handler_ovr_tlv.condition_code = (
             fault_handler_ovr_tlv.tlv.value[0] & 0xF0
         ) >> 4
@@ -267,10 +336,12 @@ class FaultHandlerOverrideTlv(ConcreteTlvBase):
 
     @classmethod
     def from_tlv(cls, cfdp_tlv: CfdpTlv) -> FaultHandlerOverrideTlv:
-        if cfdp_tlv.tlv_type != TlvTypes.FAULT_HANDLER:
-            raise ValueError
-        # This ensures that all fields are set properly
-        fault_handler_tlv = FaultHandlerOverrideTlv.unpack(raw_bytes=cfdp_tlv.pack())
+        if cfdp_tlv.tlv_type != cls.TLV_TYPE:
+            raise TlvTypeMissmatch(cfdp_tlv.tlv_type, cls.TLV_TYPE)
+        fault_handler_tlv = cls.__empty()
+        fault_handler_tlv.tlv = cfdp_tlv
+        fault_handler_tlv.condition_code = (cfdp_tlv.value[0] >> 4) & 0x0F
+        fault_handler_tlv.handler_code = cfdp_tlv.value[0] & 0x0F
         return fault_handler_tlv
 
 
@@ -280,10 +351,26 @@ def create_cfdp_proxy_and_dir_op_message_marker() -> bytes:
     return "cfdp".encode()
 
 
-class MessageToUserTlv(ConcreteTlvBase):
-    def __init__(self, value: bytes):
-        tlv = CfdpTlv(tlv_type=TlvTypes.MESSAGE_TO_USER, value=value)
-        ConcreteTlvBase.__init__(self, tlv=tlv)
+class MessageToUserTlv(AbstractTlvBase):
+    TLV_TYPE = TlvTypes.MESSAGE_TO_USER
+
+    def __init__(self, msg: bytes):
+        self.tlv = CfdpTlv(tlv_type=MessageToUserTlv.TLV_TYPE, value=msg)
+
+    def pack(self) -> bytearray:
+        return self.tlv.pack()
+
+    @property
+    def packet_len(self):
+        return self.tlv.packet_len
+
+    @property
+    def value(self) -> bytes:
+        return self.tlv.value
+
+    @property
+    def tlv_type(self) -> TlvTypes:
+        return MessageToUserTlv.TLV_TYPE
 
     def is_standard_proxy_dir_ops_msg(self) -> bool:
         if len(self.tlv.value) >= 4 and self.tlv.value[0:4].decode() == "cfdp":
@@ -292,45 +379,64 @@ class MessageToUserTlv(ConcreteTlvBase):
 
     @classmethod
     def __empty(cls):
-        return cls(value=bytes())
+        return cls(bytes())
 
     @classmethod
-    def unpack(cls, raw_bytes: bytes):
+    def unpack(cls, data: bytes) -> MessageToUserTlv:
         msg_to_user_tlv = cls.__empty()
-        msg_to_user_tlv.tlv = CfdpTlv.unpack(raw_bytes=raw_bytes)
-        msg_to_user_tlv._check_type(tlv_type=TlvTypes.MESSAGE_TO_USER)
+        msg_to_user_tlv.tlv = CfdpTlv.unpack(data)
+        msg_to_user_tlv.check_type(MessageToUserTlv.TLV_TYPE)
         return msg_to_user_tlv
 
     @classmethod
     def from_tlv(cls, cfdp_tlv: CfdpTlv) -> MessageToUserTlv:
-        if cfdp_tlv.tlv_type != TlvTypes.MESSAGE_TO_USER:
-            raise ValueError
+        if cfdp_tlv.tlv_type != cls.TLV_TYPE:
+            raise TlvTypeMissmatch(cfdp_tlv.tlv_type, cls.TLV_TYPE)
         msg_to_user_tlv = cls.__empty()
         msg_to_user_tlv.tlv = cfdp_tlv
         return msg_to_user_tlv
 
 
-class FlowLabelTlv(ConcreteTlvBase):
-    def __init__(self, value: bytes):
-        tlv = CfdpTlv(tlv_type=TlvTypes.FLOW_LABEL, value=value)
-        ConcreteTlvBase.__init__(self, tlv=tlv)
+class FlowLabelTlv(AbstractTlvBase):
+    TLV_TYPE = TlvTypes.FLOW_LABEL
+
+    def __init__(self, flow_label: bytes):
+        self.tlv = CfdpTlv(tlv_type=self.tlv_type, value=flow_label)
 
     @classmethod
     def __empty(cls):
-        return cls(value=bytes())
+        return cls(bytes())
+
+    def pack(self) -> bytearray:
+        return self.tlv.pack()
+
+    @property
+    def packet_len(self):
+        return self.tlv.packet_len
+
+    @property
+    def value(self) -> bytes:
+        return self.tlv.value
+
+    @property
+    def tlv_type(self) -> TlvTypes:
+        return FlowLabelTlv.TLV_TYPE
 
     @classmethod
-    def unpack(cls, raw_bytes: bytes) -> FlowLabelTlv:
+    def unpack(cls, data: bytes) -> FlowLabelTlv:
         flow_label_tlv = cls.__empty()
-        flow_label_tlv.tlv = CfdpTlv.unpack(raw_bytes=raw_bytes)
-        flow_label_tlv._check_type(tlv_type=TlvTypes.FLOW_LABEL)
+        tlv = CfdpTlv.unpack(data=data)
+        if tlv.tlv_type != FlowLabelTlv.TLV_TYPE:
+            raise TlvTypeMissmatch(tlv.tlv_type, cls.TLV_TYPE)
+        flow_label_tlv.tlv = tlv
         return flow_label_tlv
 
     @classmethod
     def from_tlv(cls, cfdp_tlv: CfdpTlv) -> FlowLabelTlv:
+        if cfdp_tlv.tlv_type != FlowLabelTlv.TLV_TYPE:
+            raise TlvTypeMissmatch(cfdp_tlv.tlv_type, cls.TLV_TYPE)
         flow_label_tlv = cls.__empty()
         flow_label_tlv.tlv = cfdp_tlv
-        flow_label_tlv._check_type(tlv_type=TlvTypes.FLOW_LABEL)
         return flow_label_tlv
 
 
@@ -344,6 +450,7 @@ class FileStoreRequestBase:
         self.action_code = action_code
         self.first_file_name = first_file_name
         self.second_file_name = second_file_name
+        self.tlv: Optional[CfdpTlv] = None
 
     def _common_packer(self, status_code: int) -> bytearray:
         tlv_value = bytearray()
@@ -359,9 +466,29 @@ class FileStoreRequestBase:
             tlv_value.extend(second_name_lv.pack())
         return tlv_value
 
+    def common_packet_len(self) -> int:
+        # 2 bytes TLV header, 1 byte action code and status code, first file name LV length
+        expected_len = 3 + len(self.first_file_name) + 1
+        if self.action_code in [
+            FilestoreActionCode.REPLACE_FILE_SNP,
+            FilestoreActionCode.RENAME_FILE_SNP,
+            FilestoreActionCode.APPEND_FILE_SNP,
+        ]:
+            expected_len += len(self.second_file_name) + 1
+        return expected_len
+
+    @staticmethod
+    def _check_raw_tlv_field(first_byte: int, expected: TlvTypes):
+        try:
+            raw_tlv_type = TlvTypes(first_byte)
+            if raw_tlv_type != expected:
+                raise TlvTypeMissmatch(raw_tlv_type, expected)
+        except IndexError:
+            raise ValueError(f"No TLV type for raw field {first_byte}")
+
     @staticmethod
     def _common_unpacker(
-        raw_bytes: bytes, expected_tlv_type: TlvTypes
+        raw_bytes: bytes,
     ) -> Tuple[FilestoreActionCode, str, int, int, Optional[str]]:
         """Does only unpack common fields, does not unpack the filestore message of a Filestore
         Response package
@@ -370,24 +497,17 @@ class FileStoreRequestBase:
             is the first file name, the second value is the status code as an integer,
             the third value is the length of the full TLV packet
         """
-        tlv = CfdpTlv.unpack(raw_bytes=raw_bytes)
-        if tlv.tlv_type != expected_tlv_type:
-            logger = get_console_logger()
-            logger.warning("Invalid TLV type detected")
-            raise ValueError
         value_idx = 0
-        action_code_as_int = (tlv.value[value_idx] & 0xF0) >> 4
+        action_code_as_int = (raw_bytes[value_idx] >> 4) & 0x0F
         try:
             action_code = FilestoreActionCode(action_code_as_int)
         except ValueError:
-            logger = get_console_logger()
-            logger.warning(
+            raise ValueError(
                 f"Invalid action code in file store response with value {action_code_as_int}"
             )
-            raise ValueError
-        status_code_as_int = tlv.value[value_idx] & 0x0F
+        status_code_as_int = raw_bytes[value_idx] & 0x0F
         value_idx += 1
-        first_lv = CfdpLv.unpack(raw_bytes=tlv.value[value_idx:])
+        first_lv = CfdpLv.unpack(raw_bytes=raw_bytes[value_idx:])
         value_idx += first_lv.packet_len
         first_file_name = first_lv.value.decode()
         if action_code in [
@@ -395,7 +515,7 @@ class FileStoreRequestBase:
             FilestoreActionCode.RENAME_FILE_SNP,
             FilestoreActionCode.APPEND_FILE_SNP,
         ]:
-            second_lv = CfdpLv.unpack(raw_bytes=tlv.value[value_idx:])
+            second_lv = CfdpLv.unpack(raw_bytes=raw_bytes[value_idx:])
             value_idx += second_lv.packet_len
             second_file_name = second_lv.value.decode()
         else:
@@ -404,12 +524,14 @@ class FileStoreRequestBase:
             action_code,
             first_file_name,
             status_code_as_int,
-            tlv.packet_len,
+            value_idx,
             second_file_name,
         )
 
 
-class FileStoreRequestTlv(FileStoreRequestBase, ConcreteTlvBase):
+class FileStoreRequestTlv(FileStoreRequestBase, AbstractTlvBase):
+    TLV_TYPE = TlvTypes.FILESTORE_REQUEST
+
     def __init__(
         self,
         action_code: FilestoreActionCode,
@@ -421,8 +543,27 @@ class FileStoreRequestTlv(FileStoreRequestBase, ConcreteTlvBase):
             first_file_name=first_file_name,
             second_file_name=second_file_name,
         )
-        tlv = self._build_tlv()
-        ConcreteTlvBase.__init__(self, tlv=tlv)
+
+    def generate_tlv(self):
+        if self.tlv is None:
+            self.tlv = self._build_tlv()
+
+    def pack(self) -> bytearray:
+        self.generate_tlv()
+        return self.tlv.pack()
+
+    @property
+    def packet_len(self):
+        return self.common_packet_len()
+
+    @property
+    def value(self) -> bytes:
+        self.generate_tlv()
+        return self.tlv.value
+
+    @property
+    def tlv_type(self) -> TlvTypes:
+        return FileStoreRequestTlv.TLV_TYPE
 
     @classmethod
     def __empty(cls) -> FileStoreRequestTlv:
@@ -437,28 +578,35 @@ class FileStoreRequestTlv(FileStoreRequestBase, ConcreteTlvBase):
         return CfdpTlv(tlv_type=TlvTypes.FILESTORE_REQUEST, value=tlv_value)
 
     @classmethod
-    def unpack(cls, raw_bytes: bytearray) -> FileStoreRequestTlv:
+    def unpack(cls, data: bytes) -> FileStoreRequestTlv:
+        cls._check_raw_tlv_field(data[0], FileStoreRequestTlv.TLV_TYPE)
         filestore_req = cls.__empty()
-        action_code, first_name, status_code, _, second_name = cls._common_unpacker(
-            raw_bytes=raw_bytes, expected_tlv_type=TlvTypes.FILESTORE_REQUEST
-        )
-        filestore_req.action_code = action_code
-        filestore_req.first_file_name = first_name
-        if second_name is not None:
-            filestore_req.second_file_name = second_name
-        filestore_req.tlv = filestore_req._build_tlv()
+        cls._set_fields(filestore_req, data[2:])
         return filestore_req
 
     @classmethod
     def from_tlv(cls, cfdp_tlv: CfdpTlv) -> FileStoreRequestTlv:
-        if cfdp_tlv.tlv_type != TlvTypes.FILESTORE_REQUEST:
-            raise ValueError
-        # This ensures that all fields are set properly, although its not the most efficient way
-        fault_handler_tlv = FileStoreRequestTlv.unpack(raw_bytes=cfdp_tlv.pack())
-        return fault_handler_tlv
+        if cfdp_tlv.tlv_type != cls.TLV_TYPE:
+            raise TlvTypeMissmatch(cfdp_tlv.tlv_type, cls.TLV_TYPE)
+        fs_response = cls.__empty()
+        cls._set_fields(fs_response, cfdp_tlv.value)
+        return fs_response
+
+    @classmethod
+    def _set_fields(cls, instance: FileStoreRequestTlv, raw_data: bytes):
+        action_code, first_name, status_code, _, second_name = cls._common_unpacker(
+            raw_bytes=raw_data
+        )
+        instance.action_code = action_code
+        instance.first_file_name = first_name
+        if second_name is not None:
+            instance.second_file_name = second_name
+        return instance
 
 
-class FileStoreResponseTlv(FileStoreRequestBase, ConcreteTlvBase):
+class FileStoreResponseTlv(FileStoreRequestBase, AbstractTlvBase):
+    TLV_TYPE = TlvTypes.FILESTORE_RESPONSE
+
     def __init__(
         self,
         action_code: FilestoreActionCode,
@@ -474,8 +622,27 @@ class FileStoreResponseTlv(FileStoreRequestBase, ConcreteTlvBase):
         )
         self.status_code = status_code
         self.filestore_msg = filestore_msg
-        tlv = self._build_tlv()
-        ConcreteTlvBase.__init__(self, tlv=tlv)
+
+    def generate_tlv(self):
+        if self.tlv is None:
+            self.tlv = self._build_tlv()
+
+    def pack(self) -> bytearray:
+        self.generate_tlv()
+        return self.tlv.pack()
+
+    @property
+    def value(self) -> bytes:
+        self.generate_tlv()
+        return self.tlv.value
+
+    @property
+    def packet_len(self):
+        return self.common_packet_len() + self.filestore_msg.packet_len
+
+    @property
+    def tlv_type(self) -> TlvTypes:
+        return FileStoreResponseTlv.TLV_TYPE
 
     @classmethod
     def __empty(cls) -> FileStoreResponseTlv:
@@ -493,65 +660,90 @@ class FileStoreResponseTlv(FileStoreRequestBase, ConcreteTlvBase):
         return CfdpTlv(tlv_type=TlvTypes.FILESTORE_RESPONSE, value=tlv_value)
 
     @classmethod
-    def unpack(cls, raw_bytes: bytearray) -> FileStoreResponseTlv:
+    def unpack(cls, data: bytes) -> FileStoreResponseTlv:
+        cls._check_raw_tlv_field(data[0], FileStoreResponseTlv.TLV_TYPE)
         filestore_reply = cls.__empty()
+        cls._set_fields(filestore_reply, data[2:])
+        return filestore_reply
+
+    @classmethod
+    def from_tlv(cls, cfdp_tlv: CfdpTlv) -> FileStoreResponseTlv:
+        if cfdp_tlv.tlv_type != cls.TLV_TYPE:
+            raise TlvTypeMissmatch(cfdp_tlv.tlv_type, cls.TLV_TYPE)
+        fs_response = FileStoreResponseTlv.__empty()
+        cls._set_fields(fs_response, cfdp_tlv.value)
+        return fs_response
+
+    @classmethod
+    def _set_fields(cls, instance: FileStoreResponseTlv, data: bytes):
         action_code, first_name, status_code, idx, second_name = cls._common_unpacker(
-            raw_bytes=raw_bytes, expected_tlv_type=TlvTypes.FILESTORE_RESPONSE
+            raw_bytes=data
         )
-        filestore_reply.action_code = action_code
-        filestore_reply.first_file_name = first_name
+        instance.action_code = action_code
+        instance.first_file_name = first_name
         try:
             status_code_named = FilestoreResponseStatusCode(
                 action_code << 4 | status_code
             )
         except ValueError:
-            logger = get_console_logger()
-            logger.warning(
+            raise ValueError(
                 f"Invalid status code in file store response with value {status_code} for "
                 f"action code {action_code}"
             )
-            raise ValueError
-        filestore_reply.status_code = status_code_named
+        instance.status_code = status_code_named
         if second_name is not None:
-            filestore_reply.second_file_name = second_name
-        filestore_reply.filestore_msg = CfdpLv.unpack(raw_bytes=raw_bytes[idx - 1 :])
-        filestore_reply.tlv = filestore_reply._build_tlv()
-        return filestore_reply
-
-    @classmethod
-    def from_tlv(cls, cfdp_tlv: CfdpTlv) -> FileStoreResponseTlv:
-        if cfdp_tlv.tlv_type != TlvTypes.FILESTORE_RESPONSE:
-            raise ValueError
-        # This ensures that all fields are set properly, although its not the most efficient way
-        file_store_response_tlv = FileStoreResponseTlv.unpack(raw_bytes=cfdp_tlv.pack())
-        return file_store_response_tlv
+            instance.second_file_name = second_name
+        instance.filestore_msg = CfdpLv.unpack(data[idx:])
 
 
-TlvBase = TypeVar("TlvBase", bound=ConcreteTlvBase)
-TlvList = List[Union[CfdpTlv, TlvBase]]
+TlvList = List[Union[AbstractTlvBase]]
 
 
-def concrete_tlv_factory(
-    cfdp_tlv: CfdpTlv, _tlv_type: Type[TlvBase] = TlvBase
-) -> TlvBase:
-    """This factory returns the concrete CFDP TLV class from a generic CFDP TLV.
+class TlvHolder:
+    def __init__(self, tlv_base: Optional[AbstractTlvBase]):
+        self.base = tlv_base
 
-    Please note that this might not be the most efficient way to create the concrete classes and
-    it might be more efficient to deserialize a raw buffer into the concrete class directly after
-    checking the TLV type
-    :param cfdp_tlv: Generic TLV packet
-    :param _tlv_type: Optional type specification of the concrete class. This is useful for IDEs
-        and type checkers
-    """
-    if cfdp_tlv.tlv_type == TlvTypes.FILESTORE_REQUEST:
-        return FileStoreRequestTlv.from_tlv(cfdp_tlv=cfdp_tlv)
-    elif cfdp_tlv.tlv_type == TlvTypes.FILESTORE_RESPONSE:
-        return FileStoreResponseTlv.from_tlv(cfdp_tlv=cfdp_tlv)
-    elif cfdp_tlv.tlv_type == TlvTypes.MESSAGE_TO_USER:
-        return MessageToUserTlv.from_tlv(cfdp_tlv=cfdp_tlv)
-    elif cfdp_tlv.tlv_type == TlvTypes.FAULT_HANDLER:
-        return FaultHandlerOverrideTlv.from_tlv(cfdp_tlv=cfdp_tlv)
-    elif cfdp_tlv.tlv_type == TlvTypes.FLOW_LABEL:
-        return FlowLabelTlv.from_tlv(cfdp_tlv=cfdp_tlv)
-    elif cfdp_tlv.tlv_type == TlvTypes.ENTITY_ID:
-        return EntityIdTlv.from_tlv(cfdp_tlv=cfdp_tlv)
+    @property
+    def tlv_type(self):
+        if self.base is not None:
+            return self.base.tlv_type
+
+    def __cast_internally(
+        self,
+        obj_type: Type[AbstractTlvBase],
+        expected_type: TlvTypes,
+    ) -> Any:
+        if self.base.tlv_type != expected_type:
+            raise TypeError(f"Invalid object {self.base} for type {self.base.tlv_type}")
+        return cast(obj_type, self.base)
+
+    def to_fs_request(self) -> FileStoreRequestTlv:
+        # Check this type first. It's a concrete type where we can not just use a simple cast
+        if isinstance(self.base, CfdpTlv):
+            return FileStoreRequestTlv.from_tlv(self.base)
+        return self.__cast_internally(FileStoreRequestTlv, TlvTypes.FILESTORE_REQUEST)
+
+    def to_fs_response(self) -> FileStoreResponseTlv:
+        if isinstance(self.base, CfdpTlv):
+            return FileStoreResponseTlv.from_tlv(self.base)
+        return self.__cast_internally(FileStoreResponseTlv, TlvTypes.FILESTORE_RESPONSE)
+
+    def to_msg_to_user(self) -> MessageToUserTlv:
+        if isinstance(self.base, CfdpTlv):
+            return MessageToUserTlv.from_tlv(self.base)
+        return self.__cast_internally(MessageToUserTlv, TlvTypes.MESSAGE_TO_USER)
+
+    def to_fault_handler_override(self) -> FaultHandlerOverrideTlv:
+        if isinstance(self.base, CfdpTlv):
+            return FaultHandlerOverrideTlv.from_tlv(self.base)
+        return self.__cast_internally(FaultHandlerOverrideTlv, TlvTypes.FAULT_HANDLER)
+
+    def to_flow_label(self) -> FlowLabelTlv:
+        if isinstance(self.base, CfdpTlv):
+            return FlowLabelTlv.from_tlv(self.base)
+        return self.__cast_internally(FlowLabelTlv, TlvTypes.FLOW_LABEL)
+
+    def to_entity_id(self) -> EntityIdTlv:
+        if isinstance(self.base, CfdpTlv):
+            return EntityIdTlv.from_tlv(self.base)
+        return self.__cast_internally(EntityIdTlv, TlvTypes.ENTITY_ID)
