@@ -3,21 +3,23 @@
 from __future__ import annotations
 
 import enum
-import struct
 from dataclasses import dataclass
 from typing import Optional
 
-from spacepackets.ccsds.spacepacket import PacketId, PacketSeqCtrl, SpacePacketHeader
-from spacepackets.ccsds.time import CdsShortTimestamp
+from spacepackets.ccsds import SpacePacketHeader
+from spacepackets.ccsds.time import CcsdsTimeProvider
 from spacepackets.ecss import PusTelecommand
 from spacepackets.ecss.conf import FETCH_GLOBAL_APID
-from spacepackets.ecss.defs import PusServices
+from spacepackets.ecss.defs import PusService
 from spacepackets.ecss.fields import PacketFieldEnum
-from spacepackets.ecss.tm import PusTelemetry
-from spacepackets.log import get_console_logger
+from spacepackets.ecss.tm import PusTelemetry, AbstractPusTm
+from .exceptions import TmSrcDataTooShortError
+
+from .req_id import RequestId
+from .. import BytesTooShortError
 
 
-class Subservices(enum.IntEnum):
+class Subservice(enum.IntEnum):
     INVALID = 0
     TM_ACCEPTANCE_SUCCESS = 1
     TM_ACCEPTANCE_FAILURE = 2
@@ -29,77 +31,12 @@ class Subservices(enum.IntEnum):
     TM_COMPLETION_FAILURE = 8
 
 
-class RequestId:
-    def __init__(
-        self, tc_packet_id: PacketId, tc_psc: PacketSeqCtrl, ccsds_version: int = 0b000
-    ):
-        self.tc_packet_id = tc_packet_id
-        self.tc_psc = tc_psc
-        self.ccsds_version = ccsds_version
-
-    @classmethod
-    def empty(cls):
-        return cls(PacketId.empty(), PacketSeqCtrl.empty())
-
-    @classmethod
-    def unpack(cls, tm_data: bytes) -> RequestId:
-        if len(tm_data) < 4:
-            raise ValueError(
-                "Given Raw TM data too small to parse Request ID. Must be 4 bytes at least"
-            )
-        packet_id_version_raw = struct.unpack("!H", tm_data[0:2])[0]
-        psc_raw = struct.unpack("!H", tm_data[2:4])[0]
-        return cls(
-            ccsds_version=(packet_id_version_raw >> 13) & 0b111,
-            tc_packet_id=PacketId.from_raw(packet_id_version_raw),
-            tc_psc=PacketSeqCtrl.from_raw(psc_raw),
-        )
-
-    @classmethod
-    def from_pus_tc(cls, pus_tc: PusTelecommand):
-        return cls.from_sp_header(pus_tc.sp_header)
-
-    @classmethod
-    def from_sp_header(cls, header: SpacePacketHeader) -> RequestId:
-        return cls(
-            ccsds_version=header.ccsds_version,
-            tc_packet_id=header.packet_id,
-            tc_psc=header.psc,
-        )
-
-    def pack(self) -> bytes:
-        raw = bytearray()
-        packet_id_and_version = (self.ccsds_version << 13) | self.tc_packet_id.raw()
-        raw.extend(struct.pack("!H", packet_id_and_version))
-        raw.extend(struct.pack("!H", self.tc_psc.raw()))
-        return raw
-
-    def _as_u32(self):
-        packet_id_and_version = (self.ccsds_version << 13) | self.tc_packet_id.raw()
-        return (packet_id_and_version << 16) | self.tc_psc.raw()
-
-    def __repr__(self):
-        return (
-            f"{self.__class__.__name__}(tc_packet_id={self.tc_packet_id!r}, "
-            f"tc_psc={self.tc_psc!r}, ccsds_version={self.ccsds_version!r})"
-        )
-
-    def __eq__(self, other: RequestId):
-        return self._as_u32() == other._as_u32()
-
-    def __hash__(self):
-        return self._as_u32().__hash__()
-
-
 ErrorCode = PacketFieldEnum
 
 
 class FailureNotice:
     def __init__(self, code: ErrorCode, data: bytes):
-        if (code.pfc % 8) != 0:
-            raise ValueError("PFC values for error code must be byte-aligned")
-        elif round(code.pfc / 8) not in [1, 2, 4, 8]:
-            raise ValueError("Allowed byte size for failure notice: 1, 2, 4 or 8")
+        # The PFC of the passed error code is already checked for validity.
         self.code = code
         self.data = data
 
@@ -129,6 +66,7 @@ class FailureNotice:
 
 @dataclass
 class UnpackParams:
+    time_reader: Optional[CcsdsTimeProvider]
     bytes_step_id: int = 1
     bytes_err_code: int = 1
 
@@ -158,20 +96,20 @@ class VerificationParams:
             init_len += self.failure_notice.len()
         return init_len
 
-    def verify_against_subservice(self, subservice: Subservices):
+    def verify_against_subservice(self, subservice: Subservice):
         if subservice % 2 == 0:
             if self.failure_notice is None:
                 raise InvalidVerifParams("Failure Notice should be something")
-            if subservice == Subservices.TM_STEP_FAILURE and self.step_id is None:
+            if subservice == Subservice.TM_STEP_FAILURE and self.step_id is None:
                 raise InvalidVerifParams("Step ID should be something")
-            elif subservice != Subservices.TM_STEP_FAILURE and self.step_id is not None:
+            elif subservice != Subservice.TM_STEP_FAILURE and self.step_id is not None:
                 raise InvalidVerifParams("Step ID should be empty")
         else:
             if self.failure_notice is not None:
                 raise InvalidVerifParams("Failure Notice should be empty")
-            if subservice == Subservices.TM_STEP_SUCCESS and self.step_id is None:
+            if subservice == Subservice.TM_STEP_SUCCESS and self.step_id is None:
                 raise InvalidVerifParams("Step ID should be something")
-            elif subservice != Subservices.TM_STEP_SUCCESS and self.step_id is not None:
+            elif subservice != Subservice.TM_STEP_SUCCESS and self.step_id is not None:
                 raise InvalidVerifParams("Step ID should be empty")
 
 
@@ -179,18 +117,17 @@ class InvalidVerifParams(Exception):
     pass
 
 
-class Service1Tm:
-    """Service 1 TM class representation"""
+class Service1Tm(AbstractPusTm):
+    """Service 1 TM class representation."""
 
     def __init__(
         self,
-        subservice: Subservices,
+        subservice: Subservice,
+        time_provider: Optional[CcsdsTimeProvider],
         verif_params: Optional[VerificationParams] = None,
-        time: CdsShortTimestamp = None,
         seq_count: int = 0,
         apid: int = FETCH_GLOBAL_APID,
         packet_version: int = 0b000,
-        secondary_header_flag: bool = True,
         space_time_ref: int = 0b0000,
         destination_id: int = 0,
     ):
@@ -199,13 +136,12 @@ class Service1Tm:
         else:
             self._verif_params = verif_params
         self.pus_tm = PusTelemetry(
-            service=PusServices.S1_VERIFICATION,
+            service=PusService.S1_VERIFICATION,
             subservice=subservice,
-            time=time,
+            time_provider=time_provider,
             seq_count=seq_count,
             apid=apid,
             packet_version=packet_version,
-            secondary_header_flag=secondary_header_flag,
             space_time_ref=space_time_ref,
             destination_id=destination_id,
         )
@@ -217,32 +153,59 @@ class Service1Tm:
         return self.pus_tm.pack()
 
     @classmethod
-    def __empty(cls) -> Service1Tm:
-        return cls(subservice=Subservices.INVALID)
+    def __empty(cls, time_provider: Optional[CcsdsTimeProvider]) -> Service1Tm:
+        return cls(subservice=Subservice.INVALID, time_provider=time_provider)
+
+    @classmethod
+    def from_tm(cls, tm: PusTelemetry, params: UnpackParams) -> Service1Tm:
+        service_1_tm = cls.__empty(params.time_reader)
+        service_1_tm.pus_tm = tm
+        cls._unpack_raw_tm(service_1_tm, params)
+        return service_1_tm
 
     @classmethod
     def unpack(cls, data: bytes, params: UnpackParams) -> Service1Tm:
-        """Parse a service 1 telemetry packet
+        """Parse a service 1 telemetry packet.
 
         :param params:
         :param data:
-        :raises ValueError: Raw telemetry too short
+        :raises ValueError: Subservice invalid.
+        :raises BytesTooShortError: passed data too short
+        :raises TmSourceDataTooShortError: TM source data too short.
         :return:
         """
-        service_1_tm = cls.__empty()
-        service_1_tm.pus_tm = PusTelemetry.unpack(raw_telemetry=data)
+        service_1_tm = cls.__empty(params.time_reader)
+        service_1_tm.pus_tm = PusTelemetry.unpack(
+            data=data, time_reader=params.time_reader
+        )
         cls._unpack_raw_tm(service_1_tm, params)
         return service_1_tm
 
+    @property
+    def time_provider(self) -> Optional[CcsdsTimeProvider]:
+        return self.pus_tm.time_provider
+
+    @property
+    def sp_header(self) -> SpacePacketHeader:
+        return self.pus_tm.space_packet_header
+
+    @property
+    def service(self):
+        return self.pus_tm.service
+
     @property
     def subservice(self):
         return self.pus_tm.subservice
 
+    @property
+    def source_data(self) -> bytes:
+        return self.pus_tm.source_data
+
     @classmethod
     def _unpack_raw_tm(cls, instance: Service1Tm, params: UnpackParams):
         tm_data = instance.pus_tm.tm_data
         if len(tm_data) < 4:
-            raise ValueError("TM data less than 4 bytes")
+            raise TmSrcDataTooShortError(4, len(tm_data))
         instance.tc_req_id = RequestId.unpack(tm_data[0:4])
         if instance.pus_tm.subservice % 2 == 0:
             instance._unpack_failure_verification(params)
@@ -257,13 +220,9 @@ class Service1Tm:
         if subservice == 6:
             expected_len += unpack_cfg.bytes_step_id
         elif subservice not in [2, 4, 8]:
-            logger = get_console_logger()
-            logger.error("Service1TM: Invalid subservice")
+            raise ValueError(f"invalid subservice {subservice}")
         if len(tm_data) < expected_len:
-            raise ValueError(
-                f"PUS TM[1,{subservice}] source data with length {len(tm_data)} smaller than "
-                f"expected {expected_len} bytes"
-            )
+            raise TmSrcDataTooShortError(expected_len, len(tm_data))
         current_idx = 4
         if self.is_step_reply:
             self._verif_params.step_id = PacketFieldEnum.unpack(
@@ -275,14 +234,18 @@ class Service1Tm:
         )
 
     def _unpack_success_verification(self, unpack_cfg: UnpackParams):
-        if self.pus_tm.subservice == Subservices.TM_STEP_SUCCESS:
-            self._verif_params.step_id = StepId.unpack(
-                pfc=unpack_cfg.bytes_step_id * 8,
-                data=self.pus_tm.tm_data[4 : 4 + unpack_cfg.bytes_step_id],
-            )
+        if self.pus_tm.subservice == Subservice.TM_STEP_SUCCESS:
+            try:
+                self._verif_params.step_id = StepId.unpack(
+                    pfc=unpack_cfg.bytes_step_id * 8,
+                    data=self.pus_tm.tm_data[4 : 4 + unpack_cfg.bytes_step_id],
+                )
+            except BytesTooShortError as e:
+                raise TmSrcDataTooShortError(e.expected_len, e.bytes_len)
         elif self.pus_tm.subservice not in [1, 3, 7]:
-            logger = get_console_logger()
-            logger.warning("Service1TM: Invalid subservice")
+            raise ValueError(
+                f"invalid subservice {self.pus_tm.subservice}, not in [1, 3, 7]"
+            )
 
     @property
     def failure_notice(self) -> Optional[FailureNotice]:
@@ -310,8 +273,8 @@ class Service1Tm:
     @property
     def is_step_reply(self) -> bool:
         return (
-            self.subservice == Subservices.TM_STEP_FAILURE
-            or self.subservice == Subservices.TM_STEP_SUCCESS
+            self.subservice == Subservice.TM_STEP_FAILURE
+            or self.subservice == Subservice.TM_STEP_SUCCESS
         )
 
     @property
@@ -319,83 +282,113 @@ class Service1Tm:
         """Retrieve the step number. Returns NONE if this packet does not have a step ID"""
         return self._verif_params.step_id
 
+    def __eq__(self, other: Service1Tm):
+        return (self.pus_tm == other.pus_tm) and (
+            self._verif_params == other._verif_params
+        )
 
-def create_acceptance_success_tm(pus_tc: PusTelecommand) -> Service1Tm:
+
+def create_acceptance_success_tm(
+    pus_tc: PusTelecommand, time_provider: Optional[CcsdsTimeProvider]
+) -> Service1Tm:
     return Service1Tm(
-        subservice=Subservices.TM_ACCEPTANCE_SUCCESS,
+        subservice=Subservice.TM_ACCEPTANCE_SUCCESS,
         verif_params=VerificationParams(RequestId.from_sp_header(pus_tc.sp_header)),
+        time_provider=time_provider,
     )
 
 
 def create_acceptance_failure_tm(
-    pus_tc: PusTelecommand, failure_notice: FailureNotice
+    pus_tc: PusTelecommand,
+    failure_notice: FailureNotice,
+    time_provider: Optional[CcsdsTimeProvider],
 ) -> Service1Tm:
     return Service1Tm(
-        subservice=Subservices.TM_ACCEPTANCE_FAILURE,
+        subservice=Subservice.TM_ACCEPTANCE_FAILURE,
         verif_params=VerificationParams(
             req_id=RequestId.from_sp_header(pus_tc.sp_header),
             failure_notice=failure_notice,
         ),
+        time_provider=time_provider,
     )
 
 
-def create_start_success_tm(pus_tc: PusTelecommand) -> Service1Tm:
+def create_start_success_tm(
+    pus_tc: PusTelecommand, time_provider: Optional[CcsdsTimeProvider]
+) -> Service1Tm:
     return Service1Tm(
-        subservice=Subservices.TM_START_SUCCESS,
+        subservice=Subservice.TM_START_SUCCESS,
         verif_params=VerificationParams(RequestId.from_sp_header(pus_tc.sp_header)),
+        time_provider=time_provider,
     )
 
 
 def create_start_failure_tm(
-    pus_tc: PusTelecommand, failure_notice: FailureNotice
+    pus_tc: PusTelecommand,
+    failure_notice: FailureNotice,
+    time_provider: Optional[CcsdsTimeProvider],
 ) -> Service1Tm:
     return Service1Tm(
-        subservice=Subservices.TM_START_FAILURE,
+        subservice=Subservice.TM_START_FAILURE,
         verif_params=VerificationParams(
             req_id=RequestId.from_sp_header(pus_tc.sp_header),
             failure_notice=failure_notice,
         ),
+        time_provider=time_provider,
     )
 
 
 def create_step_success_tm(
-    pus_tc: PusTelecommand, step_id: PacketFieldEnum
+    pus_tc: PusTelecommand,
+    step_id: PacketFieldEnum,
+    time_provider: Optional[CcsdsTimeProvider],
 ) -> Service1Tm:
     return Service1Tm(
-        subservice=Subservices.TM_STEP_SUCCESS,
+        subservice=Subservice.TM_STEP_SUCCESS,
         verif_params=VerificationParams(
             req_id=RequestId.from_sp_header(pus_tc.sp_header), step_id=step_id
         ),
+        time_provider=time_provider,
     )
 
 
 def create_step_failure_tm(
-    pus_tc: PusTelecommand, step_id: PacketFieldEnum, failure_notice: FailureNotice
+    pus_tc: PusTelecommand,
+    step_id: PacketFieldEnum,
+    failure_notice: FailureNotice,
+    time_provider: Optional[CcsdsTimeProvider],
 ) -> Service1Tm:
     return Service1Tm(
-        subservice=Subservices.TM_STEP_FAILURE,
+        subservice=Subservice.TM_STEP_FAILURE,
         verif_params=VerificationParams(
             req_id=RequestId.from_sp_header(pus_tc.sp_header),
             step_id=step_id,
             failure_notice=failure_notice,
         ),
+        time_provider=time_provider,
     )
 
 
-def create_completion_success_tm(pus_tc: PusTelecommand) -> Service1Tm:
+def create_completion_success_tm(
+    pus_tc: PusTelecommand, time_provider: Optional[CcsdsTimeProvider]
+) -> Service1Tm:
     return Service1Tm(
-        subservice=Subservices.TM_COMPLETION_SUCCESS,
+        subservice=Subservice.TM_COMPLETION_SUCCESS,
         verif_params=VerificationParams(RequestId.from_sp_header(pus_tc.sp_header)),
+        time_provider=time_provider,
     )
 
 
 def create_completion_failure_tm(
-    pus_tc: PusTelecommand, failure_notice: FailureNotice
+    pus_tc: PusTelecommand,
+    failure_notice: FailureNotice,
+    time_provider: Optional[CcsdsTimeProvider],
 ) -> Service1Tm:
     return Service1Tm(
-        subservice=Subservices.TM_COMPLETION_FAILURE,
+        subservice=Subservice.TM_COMPLETION_FAILURE,
         verif_params=VerificationParams(
             req_id=RequestId.from_sp_header(pus_tc.sp_header),
             failure_notice=failure_notice,
         ),
+        time_provider=time_provider,
     )
