@@ -1,37 +1,32 @@
-"""This module contains the PUS telecommand class representation to pack telecommands.
+"""This module contains the PUS telecommand class representation to pack telecommands, most notably
+the :py:class:`PusTelecommand` class.
 """
 from __future__ import annotations
 
+from spacepackets import __version__, BytesTooShortError
 import struct
-import sys
-from typing import Tuple
+from typing import Tuple, Optional
+
+import deprecation
+from spacepackets.ecss.crc import CRC16_CCITT_FUNC
+from crcmod.predefined import PredefinedCrc
 
-from spacepackets.log import get_console_logger
 from spacepackets.ccsds.spacepacket import (
     SpacePacketHeader,
-    PacketTypes,
+    PacketType,
     SPACE_PACKET_HEADER_SIZE,
     SpacePacket,
     PacketId,
     PacketSeqCtrl,
     SequenceFlags,
 )
-from spacepackets.util import get_printable_data_string, PrintFormats
 from spacepackets.ecss.conf import (
     get_default_tc_apid,
     PusVersion,
-    get_max_tc_packet_size,
     FETCH_GLOBAL_APID,
 )
 
 
-try:
-    from crcmod.predefined import mkPredefinedCrcFun, PredefinedCrc
-except ImportError:
-    print("crcmod package not installed!")
-    sys.exit(1)
-
-
 class PusTcDataFieldHeader:
     PUS_C_SEC_HEADER_LEN = 5
 
@@ -64,25 +59,24 @@ class PusTcDataFieldHeader:
         return header_raw
 
     @classmethod
-    def unpack(cls, raw_packet: bytes) -> PusTcDataFieldHeader:
+    def unpack(cls, data: bytes) -> PusTcDataFieldHeader:
         """Unpack a TC data field header.
 
-        :param raw_packet: Start of raw data belonging to the TC data field header
+        :param data: Start of raw data belonging to the TC data field header
+        :raises BytesTooShortError: Passed data too short.
         :return:
         """
         min_expected_len = cls.get_header_size()
-        if len(raw_packet) < min_expected_len:
-            raise ValueError(
-                f"Passed bytearray too short, expected minimum length {min_expected_len}"
-            )
-        version_and_ack_byte = raw_packet[0]
+        if len(data) < min_expected_len:
+            raise BytesTooShortError(min_expected_len, len(data))
+        version_and_ack_byte = data[0]
         pus_version = (version_and_ack_byte & 0xF0) >> 4
         if pus_version != PusVersion.PUS_C:
             raise ValueError("This implementation only supports PUS C")
         ack_flags = version_and_ack_byte & 0x0F
-        service = raw_packet[1]
-        subservice = raw_packet[2]
-        source_id = struct.unpack("!H", raw_packet[3:5])[0]
+        service = data[1]
+        subservice = data[2]
+        source_id = struct.unpack("!H", data[3:5])[0]
         return cls(
             service=service,
             subservice=subservice,
@@ -104,10 +98,23 @@ class PusTcDataFieldHeader:
         return cls.PUS_C_SEC_HEADER_LEN
 
 
+class InvalidTcCrc16(Exception):
+    def __init__(self, tc: PusTelecommand):
+        self.tc = tc
+
+
 class PusTelecommand:
-    """Class representation of a PUS telecommand. It can be used to pack a raw telecommand from
-    input parameters. The structure of a PUS telecommand is specified in ECSS-E-70-41A on p.42
-    and is also shown below (bottom)
+    """Class representation of a PUS telecommand. Can be converted to the raw byte representation
+    but also unpacked from a raw byte stream. Only PUS C telecommands are supported.
+
+    >>> ping_tc = PusTelecommand(service=17, subservice=1, seq_count=22, apid=0x01)
+    >>> ping_tc.service
+    17
+    >>> ping_tc.subservice
+    1
+    >>> ping_tc.pack().hex(sep=',')
+    '18,01,c0,16,00,06,2f,11,01,00,00,ab,62'
+
     """
 
     def __init__(
@@ -115,11 +122,10 @@ class PusTelecommand:
         service: int,
         subservice: int,
         app_data: bytes = bytes([]),
+        apid: int = FETCH_GLOBAL_APID,
         seq_count: int = 0,
-        seq_flags: SequenceFlags = SequenceFlags.UNSEGMENTED,
         source_id: int = 0,
         ack_flags: int = 0b1111,
-        apid: int = FETCH_GLOBAL_APID,
     ):
         """Initiate a PUS telecommand from the given parameters. The raw byte representation
         can then be retrieved with the :py:meth:`pack` function.
@@ -134,14 +140,8 @@ class PusTelecommand:
             different packet sources (e.g. different ground stations)
         :raises ValueError: Invalid input parameters
         """
-        if apid == -1:
+        if apid == FETCH_GLOBAL_APID:
             apid = get_default_tc_apid()
-        secondary_header_flag = 1
-        logger = get_console_logger()
-        if len(app_data) > get_max_tc_packet_size():
-            logger.warning(
-                "Application data of PUS packet exceeds maximum allowed size"
-            )
         self.pus_tc_sec_header = PusTcDataFieldHeader(
             service=service,
             subservice=subservice,
@@ -154,15 +154,15 @@ class PusTelecommand:
         )
         self.sp_header = SpacePacketHeader(
             apid=apid,
-            sec_header_flag=bool(secondary_header_flag),
-            packet_type=PacketTypes.TC,
-            seq_flags=seq_flags,
+            sec_header_flag=True,
+            packet_type=PacketType.TC,
+            seq_flags=SequenceFlags.UNSEGMENTED,
             data_len=data_length,
             seq_count=seq_count,
         )
         self._app_data = app_data
         self._valid = True
-        self._crc16 = 0
+        self._crc16: Optional[bytes] = None
 
     @classmethod
     def from_sp_header(
@@ -174,11 +174,13 @@ class PusTelecommand:
         source_id: int = 0,
         ack_flags: int = 0b1111,
     ):
-        pus_tc = cls.__empty()
-        if sp_header.packet_type == PacketTypes.TM:
-            raise ValueError(
-                f"Invalid Packet Type {sp_header.packet_type} in CCSDS primary header"
-            )
+        pus_tc = cls.empty()
+        sp_header.packet_type = PacketType.TC
+        sp_header.sec_header_flag = True
+        sp_header.data_len = PusTelecommand.get_data_length(
+            secondary_header_len=PusTcDataFieldHeader.get_header_size(),
+            app_data_len=len(app_data),
+        )
         pus_tc.sp_header = sp_header
         pus_tc.pus_tc_sec_header = PusTcDataFieldHeader(
             service=service,
@@ -196,8 +198,8 @@ class PusTelecommand:
         sec_header: PusTcDataFieldHeader,
         app_data: bytes = bytes([]),
     ) -> PusTelecommand:
-        pus_tc = cls.__empty()
-        if sp_header.packet_type == PacketTypes.TM:
+        pus_tc = cls.empty()
+        if sp_header.packet_type == PacketType.TM:
             raise ValueError(
                 f"Invalid Packet Type {sp_header.packet_type} in CCSDS primary header"
             )
@@ -207,7 +209,7 @@ class PusTelecommand:
         return pus_tc
 
     @classmethod
-    def __empty(cls) -> PusTelecommand:
+    def empty(cls) -> PusTelecommand:
         return PusTelecommand(service=0, subservice=0)
 
     def __repr__(self):
@@ -219,10 +221,12 @@ class PusTelecommand:
 
     def __str__(self):
         """Returns string representation of a class instance."""
+        from .req_id import RequestId
+
         return (
-            f"PUS TC[{self.pus_tc_sec_header.service}, "
-            f"{self.pus_tc_sec_header.subservice}], APID {self.apid:#05x}, "
-            f"SSC {self.sp_header.seq_count}, Size {self.packet_len}"
+            f"PUS TC[{self.pus_tc_sec_header.service}, {self.pus_tc_sec_header.subservice}] with "
+            f"Request ID {RequestId.from_sp_header(self.sp_header).as_u32():#08x}"
+            f", APID {self.apid:#05x}, SSC {self.sp_header.seq_count}"
         )
 
     def __eq__(self, other: PusTelecommand):
@@ -237,65 +241,56 @@ class PusTelecommand:
         before converting the PUS TC to a generic Space Packet"""
         self.calc_crc()
         user_data = bytearray(self._app_data)
-        user_data.extend(struct.pack("!H", self._crc16))
+        user_data.extend(self._crc16)
         return SpacePacket(self.sp_header, self.pus_tc_sec_header.pack(), user_data)
 
-    @property
-    def valid(self):
-        return self._valid
-
     def calc_crc(self):
-        """Can be called to calculate the CRC16"""
+        """Can be called to calculate the CRC16. Also sets the internal CRC16 field."""
         crc = PredefinedCrc(crc_name="crc-ccitt-false")
         crc.update(self.sp_header.pack())
         crc.update(self.pus_tc_sec_header.pack())
         crc.update(self.app_data)
-        self._crc16 = crc.crcValue
+        self._crc16 = struct.pack("!H", crc.crcValue)
 
-    def pack(self, calc_crc: bool = True) -> bytearray:
+    def pack(self, recalc_crc: bool = True) -> bytearray:
         """Serializes the TC data fields into a bytearray.
-        :param calc_crc: Recalculate the CRC. Can be disabled if :py:func:`calc_crc`
-            was called before
+
+        :param recalc_crc: Can be set to False if the CRC was previous calculated and no fields were
+            changed. This is set to True by default to ensure the CRC is always valid by default,
+            even if the user changes arbitrary fields after TC creation.
         """
         packed_data = bytearray()
         packed_data.extend(self.sp_header.pack())
         packed_data.extend(self.pus_tc_sec_header.pack())
         packed_data += self.app_data
-        if calc_crc:
-            crc_func = mkPredefinedCrcFun(crc_name="crc-ccitt-false")
-            self._crc16 = crc_func(packed_data)
-        packed_data.extend(struct.pack("!H", self._crc16))
+        if self._crc16 is None or recalc_crc:
+            self._crc16 = struct.pack("!H", CRC16_CCITT_FUNC(packed_data))
+        packed_data.extend(self._crc16)
         return packed_data
 
     @classmethod
-    def unpack(cls, raw_packet: bytes) -> PusTelecommand:
-        tc_unpacked = cls.__empty()
-        tc_unpacked.sp_header = SpacePacketHeader.unpack(space_packet_raw=raw_packet)
+    def unpack(cls, data: bytes) -> PusTelecommand:
+        """Create an instance from a raw bytestream.
+
+        :raises BytesTooShortError: Passed bytestream too short.
+        :raises ValueError: Unsupported PUS version.
+        :raises InvalidTcCrc16: Invalid CRC16.
+        """
+        tc_unpacked = cls.empty()
+        tc_unpacked.sp_header = SpacePacketHeader.unpack(data=data)
         tc_unpacked.pus_tc_sec_header = PusTcDataFieldHeader.unpack(
-            raw_packet=raw_packet[SPACE_PACKET_HEADER_SIZE:]
+            data=data[SPACE_PACKET_HEADER_SIZE:]
         )
         header_len = (
             SPACE_PACKET_HEADER_SIZE + tc_unpacked.pus_tc_sec_header.get_header_size()
         )
         expected_packet_len = tc_unpacked.packet_len
-        if len(raw_packet) < expected_packet_len:
-            logger = get_console_logger()
-            logger.warning(
-                f"Invalid length of raw telecommand packet, expected minimum length "
-                f"{expected_packet_len}"
-            )
-            raise ValueError
-        tc_unpacked._app_data = raw_packet[header_len : expected_packet_len - 2]
-        tc_unpacked._crc16 = raw_packet[expected_packet_len - 2 : expected_packet_len]
-        crc_func = mkPredefinedCrcFun(crc_name="crc-ccitt-false")
-        whole_packet = raw_packet[:expected_packet_len]
-        should_be_zero = crc_func(whole_packet)
-        if should_be_zero == 0:
-            tc_unpacked._valid = True
-        else:
-            logger = get_console_logger()
-            logger.warning("Invalid CRC16 in raw telecommand detected")
-            tc_unpacked._valid = False
+        if len(data) < expected_packet_len:
+            raise BytesTooShortError(expected_packet_len, len(data))
+        tc_unpacked._app_data = data[header_len : expected_packet_len - 2]
+        tc_unpacked._crc16 = data[expected_packet_len - 2 : expected_packet_len]
+        if CRC16_CCITT_FUNC(data[:expected_packet_len]) != 0:
+            raise InvalidTcCrc16(tc_unpacked)
         return tc_unpacked
 
     @property
@@ -314,14 +309,14 @@ class PusTelecommand:
         The size of the TC packet is the size of the packet secondary header with
         source ID + the length of the application data + length of the CRC16 checksum - 1
         """
-        try:
-            data_length = secondary_header_len + app_data_len + 1
-            return data_length
-        except TypeError:
-            logger = get_console_logger()
-            logger.warning("PusTelecommand: Invalid type of application data!")
-            return 0
+        data_length = secondary_header_len + app_data_len + 1
+        return data_length
 
+    @deprecation.deprecated(
+        deprecated_in="v0.14.0rc3",
+        current_version=__version__,
+        details="use pack and the class itself to build this instead",
+    )
     def pack_command_tuple(self) -> Tuple[bytearray, PusTelecommand]:
         """Pack a tuple consisting of the raw packet as the first entry and the class representation
         as the second entry
@@ -337,6 +332,14 @@ class PusTelecommand:
     def subservice(self) -> int:
         return self.pus_tc_sec_header.subservice
 
+    @property
+    def source_id(self) -> int:
+        return self.pus_tc_sec_header.source_id
+
+    @source_id.setter
+    def source_id(self, source_id: int):
+        self.pus_tc_sec_header.source_id = source_id
+
     @property
     def seq_count(self) -> int:
         return self.sp_header.seq_count
@@ -358,36 +361,36 @@ class PusTelecommand:
         return self._app_data
 
     @property
-    def crc16(self):
+    def crc16(self) -> Optional[bytes]:
+        """Will be the raw CRC16 if the telecommand was created using :py:meth:`unpack`,
+        :py:meth:`pack` was called at least once or :py:meth:`calc_crc` was called at
+        least once."""
         return self._crc16
 
-    def print(self, print_format: PrintFormats = PrintFormats.HEX):
-        """Print the raw command in a clean format."""
-        packet = self.pack()
-        print(
-            get_printable_data_string(
-                print_format=print_format, data=packet, length=len(packet)
-            )
-        )
+    @seq_count.setter
+    def seq_count(self, value):
+        self.sp_header.seq_count = value
+
+    @apid.setter
+    def apid(self, apid):
+        self.sp_header.apid = apid
 
 
-def generate_packet_crc(tc_packet: bytearray) -> bytearray:
+def generate_packet_crc(tc_packet: bytearray) -> bytes:
     """Removes current Packet Error Control, calculates new
     CRC16 checksum and adds it as correct Packet Error Control Code.
     Reference: ECSS-E70-41A p. 207-212
     """
-    crc_func = mkPredefinedCrcFun(crc_name="crc-ccitt-false")
-    crc = crc_func(bytearray(tc_packet[0 : len(tc_packet) - 2]))
+    crc = CRC16_CCITT_FUNC(tc_packet[0 : len(tc_packet) - 2])
     tc_packet[len(tc_packet) - 2] = (crc & 0xFF00) >> 8
     tc_packet[len(tc_packet) - 1] = crc & 0xFF
     return tc_packet
 
 
-def generate_crc(data: bytearray) -> bytearray:
+def generate_crc(data: bytearray) -> bytes:
     """Takes the application data, appends the CRC16 checksum and returns resulting bytearray"""
     data_with_crc = bytearray()
     data_with_crc += data
-    crc_func = mkPredefinedCrcFun(crc_name="crc-ccitt-false")
-    crc = crc_func(data)
+    crc = CRC16_CCITT_FUNC(data)
     data_with_crc.extend(struct.pack("!H", crc))
     return data_with_crc
